@@ -81,9 +81,9 @@ class TestSystemAPIs(unittest.TestCase):
 
     def test_wait_procs(self):
         def callback(p):
-            l.append(p.pid)
+            pids.append(p.pid)
 
-        l = []
+        pids = []
         sproc1 = get_test_subprocess()
         sproc2 = get_test_subprocess()
         sproc3 = get_test_subprocess()
@@ -96,7 +96,7 @@ class TestSystemAPIs(unittest.TestCase):
         self.assertLess(time.time() - t, 0.5)
         self.assertEqual(gone, [])
         self.assertEqual(len(alive), 3)
-        self.assertEqual(l, [])
+        self.assertEqual(pids, [])
         for p in alive:
             self.assertFalse(hasattr(p, 'returncode'))
 
@@ -115,7 +115,7 @@ class TestSystemAPIs(unittest.TestCase):
             self.assertEqual(gone.pop().returncode, -signal.SIGTERM)
         else:
             self.assertEqual(gone.pop().returncode, 1)
-        self.assertEqual(l, [sproc3.pid])
+        self.assertEqual(pids, [sproc3.pid])
         for p in alive:
             self.assertFalse(hasattr(p, 'returncode'))
 
@@ -130,7 +130,7 @@ class TestSystemAPIs(unittest.TestCase):
         sproc1.terminate()
         sproc2.terminate()
         gone, alive = test(procs, callback)
-        self.assertEqual(set(l), set([sproc1.pid, sproc2.pid, sproc3.pid]))
+        self.assertEqual(set(pids), set([sproc1.pid, sproc2.pid, sproc3.pid]))
         for p in gone:
             self.assertTrue(hasattr(p, 'returncode'))
 
@@ -454,6 +454,10 @@ class TestSystemAPIs(unittest.TestCase):
         # AssertionError: Lists differ: [0, 1, 2, 3, 4, 5, 6, 7,... != [0]
         self.assertTrue(ls, msg=ls)
         for disk in ls:
+            self.assertIsInstance(disk.device, (str, unicode))
+            self.assertIsInstance(disk.mountpoint, (str, unicode))
+            self.assertIsInstance(disk.fstype, (str, unicode))
+            self.assertIsInstance(disk.opts, (str, unicode))
             if WINDOWS and 'cdrom' in disk.opts:
                 continue
             if not POSIX:
@@ -468,7 +472,6 @@ class TestSystemAPIs(unittest.TestCase):
             else:
                 assert os.path.isdir(disk.mountpoint), disk
             assert disk.fstype, disk
-            self.assertIsInstance(disk.opts, str)
 
         # all = True
         ls = psutil.disk_partitions(all=True)
@@ -512,6 +515,7 @@ class TestSystemAPIs(unittest.TestCase):
                 self.assertIn(conn.family, families, msg=conn)
                 if conn.family != getattr(socket, 'AF_UNIX', object()):
                     self.assertIn(conn.type, types_, msg=conn)
+                self.assertIsInstance(conn.status, (str, unicode))
 
         from psutil._common import conn_tmap
         for kind, groups in conn_tmap.items():
@@ -547,6 +551,7 @@ class TestSystemAPIs(unittest.TestCase):
         self.assertNotEqual(ret, [])
         for key in ret:
             self.assertTrue(key)
+            self.assertIsInstance(key, (str, unicode))
             check_ntuple(ret[key])
 
     def test_net_if_addrs(self):
@@ -562,6 +567,7 @@ class TestSystemAPIs(unittest.TestCase):
 
         families = set([socket.AF_INET, AF_INET6, psutil.AF_LINK])
         for nic, addrs in nics.items():
+            self.assertIsInstance(nic, (str, unicode))
             self.assertEqual(len(set(addrs)), len(addrs))
             for addr in addrs:
                 self.assertIsInstance(addr.family, int)
@@ -684,6 +690,10 @@ class TestSystemAPIs(unittest.TestCase):
         self.assertNotEqual(users, [])
         for user in users:
             assert user.name, user
+            self.assertIsInstance(user.name, (str, unicode))
+            self.assertIsInstance(user.terminal, (str, unicode, None))
+            if user.host is not None:
+                self.assertIsInstance(user.host, (str, unicode, None))
             user.terminal
             user.host
             assert user.started > 0.0, user
@@ -757,7 +767,7 @@ class TestSystemAPIs(unittest.TestCase):
             self.assertIs(getattr(psutil, name), False, msg=name)
 
     @unittest.skipUnless(hasattr(psutil, "sensors_temperatures"),
-                         "platform not suported")
+                         "platform not supported")
     def test_sensors_temperatures(self):
         temps = psutil.sensors_temperatures()
         for name, entries in temps.items():
@@ -771,7 +781,7 @@ class TestSystemAPIs(unittest.TestCase):
                 if entry.critical is not None:
                     self.assertGreaterEqual(entry.critical, 0)
 
-    @unittest.skipUnless(LINUX or WINDOWS or FREEBSD,
+    @unittest.skipUnless(hasattr(psutil, "sensors_battery"),
                          "platform not supported")
     def test_sensors_battery(self):
         ret = psutil.sensors_battery()
@@ -787,6 +797,17 @@ class TestSystemAPIs(unittest.TestCase):
                 self.assertTrue(ret.power_plugged)
         self.assertIsInstance(ret.power_plugged, bool)
 
+    @unittest.skipUnless(hasattr(psutil, "sensors_fans"),
+                         "platform not supported")
+    def test_sensors_fans(self):
+        fans = psutil.sensors_fans()
+        for name, entries in fans.items():
+            self.assertIsInstance(name, (str, unicode))
+            for entry in entries:
+                self.assertIsInstance(entry.label, (str, unicode))
+                self.assertIsInstance(entry.current, (int, long))
+                self.assertGreaterEqual(entry.current, 0)
+
 
 if __name__ == '__main__':
     run_test_module_by_name(__file__)
